@@ -5,6 +5,7 @@ import urlparse
 from urllib import quote
 
 from zope import interface
+from zope.component import getUtility, queryUtility
 from AccessControl import ClassSecurityInfo
 
 from Products.Archetypes import atapi
@@ -12,17 +13,19 @@ from Products.ATContentTypes.content import schemata
 from Products.Archetypes.atapi import AnnotationStorage
 
 from Products.CMFCore import permissions
+from Products.CMFCore.utils import getToolByName
 
 from Products.ATContentTypes.content.link import ATLink, ATLinkSchema
+from Products.ATContentTypes.interface import IImageContent
 from Products.ATReferenceBrowserWidget import ATReferenceBrowserWidget
 
 from redturtle.smartlink import smartlinkMessageFactory as _
 from redturtle.smartlink.interfaces import ISmartLink, ISmartLinked
+from redturtle.smartlink.interfaces.utility import ILinkNormalizerUtility
 from redturtle.smartlink.config import PROJECTNAME
 
 from Products.ATContentTypes.configuration import zconf
 from Products.validation import V_REQUIRED
-from zope.component import queryUtility
 from redturtle.smartlink.interfaces.utility import ISmartlinkConfig
 
 LinkSchema = ATLinkSchema.copy() + atapi.Schema((
@@ -41,7 +44,7 @@ LinkSchema = ATLinkSchema.copy() + atapi.Schema((
               searchable=True,
               required=False,
               widget=atapi.StringWidget(
-                    label= _(u'label_smartlink_externallink', default='External Link'),
+                    label= _(u'label_smartlink_externallink', default='External link'),
                     description = _(u'help_smartlink_externallink',
                                     default=u"Enter the web address for a page which is not located on this server."),
                     i18n_domain='redturtle.smartlink',
@@ -64,11 +67,29 @@ LinkSchema = ATLinkSchema.copy() + atapi.Schema((
                     )
     ),
 
+    # ******* Advanced tab *******
+    
+    atapi.StringField('anchor',
+        required = False,
+        searchable = False,
+        default='',
+        schemata="Advanced",
+        widget = atapi.StringWidget(
+            label = _(u'label_image_anchor', default=u'Internal anchor'),
+            description = _(u'help_image_anchor',
+                            default=(u'Used only when the link is internal to the site. '
+                                     u'Use this field to obtain an internal link to a section of the target document')),
+            i18n_domain='redturtle.smartlink',
+            size = 30)
+        ),
+
+
     atapi.ImageField('image',
         required = False,
         storage = AnnotationStorage(migrate=True),
         languageIndependent = True,
         max_size = zconf.ATNewsItem.max_image_dimension,
+        schemata="Advanced",
         sizes= {'large'   : (768, 768),
                 'preview' : (400, 400),
                 'mini'    : (200, 200),
@@ -90,6 +111,7 @@ LinkSchema = ATLinkSchema.copy() + atapi.Schema((
     atapi.StringField('imageCaption',
         required = False,
         searchable = True,
+        schemata="Advanced",
         widget = atapi.StringWidget(
             description = '',
             label = _(u'label_image_caption', default=u'Image Caption'),
@@ -97,6 +119,24 @@ LinkSchema = ATLinkSchema.copy() + atapi.Schema((
             size = 40)
         ),
 
+    atapi.ImageField('favicon',
+        required = False,
+        storage = AnnotationStorage(migrate=True),
+        languageIndependent = True,
+        max_size = (16, 16),
+        schemata="Advanced",
+        validators = (('isNonEmptyFile', V_REQUIRED),
+                      ('checkNewsImageMaxSize', V_REQUIRED)),
+        widget = atapi.ImageWidget(
+            description = _(u'help_smartlink_favicon',
+                            default=(u'You can customize there the content icon. '
+                                     u'You can use this for provide the icon of the remote site')),
+            label= _(u'label_smartlink_favicon', default=u'Icon'),
+            i18n_domain='redturtle.smartlink',
+            show_content_type = False)
+        ),
+
+
 ))
 
 LinkSchema['title'].storage = atapi.AnnotationStorage()
@@ -106,7 +146,7 @@ schemata.finalizeATCTSchema(LinkSchema, moveDiscussion=False)
 
 class SmartLink(ATLink):
     """A link to an internal or external resource."""
-    interface.implements(ISmartLink)
+    interface.implements(ISmartLink, IImageContent)
 
     meta_type = "ATLink"
     schema = LinkSchema
@@ -117,6 +157,26 @@ class SmartLink(ATLink):
     
     security = ClassSecurityInfo()
 
+    security.declareProtected(permissions.View, 'size')
+    def size(self):
+        """Get size of the content
+        """
+        return self.get_size()
+
+    security.declareProtected(permissions.View, 'get_size')
+    def get_size(self):
+        """ZMI / Plone get size method
+        Size is given from the sum of the two image field
+        """
+        size = 0
+        f = self.getField('image')
+        if f is not None:
+            size+=f.get_size(self)
+        f = self.getField('favicon')
+        if f is not None:
+            size+=f.get_size(self)        
+        return size
+
     security.declareProtected(permissions.View, 'tag')
     def tag(self, **kwargs):
         """Generate image tag using the api of the ImageField
@@ -140,6 +200,12 @@ class SmartLink(ATLink):
             if image is not None and not isinstance(image, basestring):
                 # image might be None or '' for empty images
                 return image
+        elif name=='favicon':
+            field = self.getField('favicon')
+            image = field.getScale(self)
+            if image is not None and not isinstance(image, basestring):
+                # image might be None or '' for empty images
+                return image            
 
         return ATLink.__bobo_traverse__(self, REQUEST, name)
 
@@ -159,42 +225,36 @@ class SmartLink(ATLink):
         self.getField('internalLink').set(self, value, **kwargs)
         self.setRemoteUrl(self.getRemoteUrl())
 
+    security.declareProtected(permissions.View, 'getRemoteUrl')
     def getRemoteUrl(self):
         """Return the URL of the link from the appropriate field, internal or external."""
-        
-        smartlink_config = queryUtility(ISmartlinkConfig, name="smartlink_config")
-
-        # We need to check if the self object has the reference_catalog attribute. It's an integration problem
-        # with p4a that call this method when we don't have an internal link.
+        # We need to check if the self object has the reference_catalog attribute.
+        # It's an integration problem with p4a that call this method when we don't have an internal link.
         if hasattr(self, 'reference_catalog'):
             ilink = self.getInternalLink()
         else:
             ilink = None
+
+        # If we are using an internal link
         if ilink:
+            anchor = self.getAnchor() or ''
+            if anchor and not anchor.startswith("#"):
+                anchor = '#'+anchor
+            smartlink_config = queryUtility(ISmartlinkConfig, name="smartlink_config")
             if smartlink_config:
                 if smartlink_config.relativelink:            
                     object = self.getField('internalLink').get(self)
                     remote = '/'.join(object.getPhysicalPath())
-                    return quote(remote, safe='?$#@/:=+;$,&%')
-            remote = ilink.absolute_url()
+                    return quote(remote + anchor, safe='?$#@/:=+;$,&%')
+            remote = ilink.absolute_url() + anchor
         else:
             remote = self.getExternalLink()
 
+        remote = getUtility(ILinkNormalizerUtility).toFrontEnd(remote)
+
+        # If we still haven't remote value now, let's return the "normal" field value
         if not remote:
-            remote = ''  # ensure we have a string
-        if smartlink_config:
-            backendlinks = getattr(smartlink_config, 'backendlink', [])
-            for backendlink in backendlinks:
-                if backendlink[backendlink.__len__()-1:]=='/':
-                    blink = backendlink[:-1]
-                else:
-                    blink = backendlink
-                if remote.startswith(blink):
-                    frontendlinks = smartlink_config.frontendlink
-                    frontendlink = frontendlinks[backendlinks.index(backendlink)]
-                    if frontendlink[frontendlink.__len__()-1:]=='/':
-                        frontendlink = frontendlink[:-1]
-                    remote = remote.replace(blink,frontendlink)
+            remote = self.getField('remoteUrl').get(self)
         return quote(remote, safe='?$#@/:=+;$,&%')
 
     security.declarePrivate('cmf_edit')
@@ -203,19 +263,21 @@ class SmartLink(ATLink):
             remote_url = kwargs.get('remote_url', None)
         self.update(externalLink = remote_url, **kwargs)
 
+    security.declareProtected(permissions.View, 'post_validate')
     def post_validate(self, REQUEST, errors):
         """Check to make sure that either an internal or external link was supplied."""
-
         if not REQUEST.form.get('externalLink') and not REQUEST.form.get('internalLink'):
             xlink=REQUEST.get('externalLink', None)
             ilink=REQUEST.get('internalLink', None)
             if (not xlink and not ilink):
-                errors['externalLink'] = _("label_internallink_externallink",
-                                           default=u'You must either select an internal link or enter an external link. You cannot have both.')
+                errors['externalLink'] = _("error_externallink_internallink_required",
+                                           default=u'Please provide the external URL, or fill the "Internal link" field')
+                errors['internalLink'] = _("error_internallink_externallink_required",
+                                           default=u'Please provide the internal link, or fill the "External link" field')
             return errors
         if REQUEST.form.get('externalLink') and REQUEST.form.get('internalLink'):
-            errors['externalLink'] = _("label_internallink_externallink",
-                                       default=u'You must either select an internal link or enter an external link. You cannot have both.')
+            errors['externalLink'] = _("error_internallink_externallink_doubled",
+                                       default=u'You must select an internal link or enter an external link. You cannot have both.')
             return errors
 
     security.declarePrivate('_processForm')
@@ -228,4 +290,37 @@ class SmartLink(ATLink):
             interface.noLongerProvides(target, ISmartLinked)
         ATLink._processForm(self, data=data, metadata=metadata, REQUEST=REQUEST, values=values)
 
+    security.declareProtected(permissions.View, 'getIcon')
+    def getIcon(self, relative_to_portal=0):
+        """If a favicon was provided, show it"""
+        if not self.getFavicon():
+            return ATLink.getIcon(self, relative_to_portal)
+
+        utool = getToolByName(self, 'portal_url')
+        if relative_to_portal:
+            return self.absolute_url().replace(utool()+'/',"") + '/favicon'
+        # Relative to REQUEST['BASEPATH1']
+        res = utool(relative=1) + self.absolute_url().replace(utool(),"") + '/favicon'
+        while res[:1] == '/':
+            res = res[1:]
+        return res
+
+
+    security.declareProtected(permissions.View, 'getInternalLinkPath')
+    def getInternalLinkPath(self):
+        """Get database path to the internally linked content"""
+        ilink = self.getInternalLink()
+        if ilink:
+            return '/'.join(ilink.getPhysicalPath())
+        if not ilink and not self.getExternalLink():
+            # Try this way... this seems the only way to get the referenced object when we are deleting
+            # No event will help us... neither OFS.interfaces.IObjectWillBeRemovedEvent
+            portal_url = getToolByName(self, 'portal_url')
+            internalPath = self.getRemoteUrl().replace(getUtility(ILinkNormalizerUtility).toFrontEnd(portal_url()), '')
+            if not internalPath.startswith('/' + portal_url.getPortalObject().getId()):
+                internalPath = '/' + portal_url.getPortalObject().getId() + internalPath
+            return internalPath
+        return None
+        
+
 atapi.registerType(SmartLink, PROJECTNAME)
